@@ -952,6 +952,16 @@ class RxTAlpha(nn.Module, PyTorchModelHubMixin, pipeline_tag="text-generation",
         else:
             self.memory_attention(new_stm, attention_mask=attention_mask)
 
+    def reset_stm_state(self):
+        self.memory_attention.model.stm.reset()
+
+    def export_stm_state(self) -> torch.Tensor:
+        return self.memory_attention.model.stm.memory.clone().detach()
+
+    def load_stm_state(self, stm_state: torch.Tensor):
+        device = self.memory_attention.model.stm.memory.device
+        self.memory_attention.model.stm.update_all(stm_state.to(device))
+
     def tokenize_query(self, text: str, max_seq_len: int = 256, device: torch.device = torch.device("cpu")):
         tokenized = self.tokenizer(
             f'{self.bos_token}{self.query_token}{text}',
@@ -1000,14 +1010,19 @@ class RxTAlpha(nn.Module, PyTorchModelHubMixin, pipeline_tag="text-generation",
             'attention_mask': tokenized['attention_mask'].to(device)
         }
 
-    def stringify_token(self, token_id: int) -> str:
-        return decode_post_process(self.tokenizer.decode([token_id]))
+    def stringify_token(self, token_id: int, skip_special_tokens: bool = True, show_memory_update: bool = False) -> str:
+        if token_id == -1:
+            return '\n [STM update]' if show_memory_update else ''
+        elif token_id == -2:
+            return '\n [STM updated]' if show_memory_update else ''
+        else:
+            return decode_post_process(self.tokenizer.decode([token_id], skip_special_tokens=skip_special_tokens))
 
-    def stringify_tokens(self, generated_ids: torch.Tensor) -> list[str]:
+    def stringify_tokens(self, generated_ids: torch.Tensor, skip_special_tokens: bool = True) -> list[str]:
         decoded = []
         for token_id in generated_ids:
             # Trim after end token
-            decoded.append(decode_post_process(self.tokenizer.decode([token_id])))
+            decoded.append(decode_post_process(self.tokenizer.decode([token_id], skip_special_tokens=skip_special_tokens)))
 
         return decoded
 
@@ -1052,7 +1067,7 @@ class RxTAlpha(nn.Module, PyTorchModelHubMixin, pipeline_tag="text-generation",
 
         # Forward pass to get next token logits
         outputs = self.forward(
-            input_ids[:, -1] if not init_step else input_ids, attention_mask=attention_mask[:, -1] if not init_step else attention_mask, stm_kv_cache=stm_kv_cache,
+            input_ids[:, -1].unsqueeze(0) if not init_step else input_ids, attention_mask=attention_mask[:, -1].unsqueeze(0) if not init_step else attention_mask, stm_kv_cache=stm_kv_cache,
             use_self_attn_cache=use_self_attn_cache, action=RxTForwardAction.DECODE, current_positions=torch.tensor([[input_ids.size(-1)]]).to(input_ids.device) if not init_step else None
         )
         next_token_logits = outputs[:, -1, :]  # Get logits for next token
